// Written by retoor@molodetz.nl

// This code is a test program for the buffer manipulation functionalities provided by the "rbuffer" library.

#include "rtest.h"   // Provides test utilities and assertions for testing purposes
#include "rbuffer.h" // Provides buffer handling functions like creation, consumption and conversion

// MIT License
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

int main() {
    rtest_banner("rbuffer");
    unsigned char *content = (unsigned char *)"[   {\n\t \"\r1\t3\n4truefalsetrue \" }, ]";
    char *ignore = "\r| |\n|\t|\f|\v";
    rbuffer_t *buffer = rbuffer_new(content, ustrlen(content));
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '[');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '{');
    rbuffer_reset(buffer);
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '[');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '{');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '"');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '1');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '3');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '4');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == 't');
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == 'f');
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == 't');
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    rbuffer_pop(buffer);
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == '"');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b|}", ignore) == '}');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b", ignore) == ',');
    assert(*rbuffer_consume(buffer, "{|[|,|\"|\\d|\\b|]", ignore) == ']');
    unsigned char *str = rbuffer_to_string(buffer);
    printf(">%s<\n", str);
    free(str);
    return rtest_end("");
}